@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 """
 Matplotlib provides sophisticated date plotting capabilities, standing on the
 shoulders of python :mod:`datetime`, the add-on modules :mod:`pytz` and
@@ -40,7 +39,7 @@ assumed.  If you want to use a custom time zone, pass a
 locators you create.  See `pytz <http://pythonhosted.org/pytz/>`_ for
 information on :mod:`pytz` and timezone handling.
 
-The `dateutil module <https://dateutil.readthedocs.org>`_ provides
+The `dateutil module <https://dateutil.readthedocs.io/en/stable/>`_ provides
 additional code to handle date ticking, making it easy to place ticks
 on any kinds of dates.  See examples below.
 
@@ -76,7 +75,7 @@ Here are all the date tickers:
 
     * :class:`HourLocator`: locate hours
 
-    * :class:`DayLocator`: locate specifed days of the month
+    * :class:`DayLocator`: locate specified days of the month
 
     * :class:`WeekdayLocator`: Locate days of the week, e.g., MO, TU
 
@@ -88,9 +87,9 @@ Here are all the date tickers:
       :class:`matplotlib.dates.rrulewrapper`.  The
       :class:`rrulewrapper` is a simple wrapper around a
       :class:`dateutil.rrule` (`dateutil
-      <https://dateutil.readthedocs.org>`_) which allow almost
+      <https://dateutil.readthedocs.io/en/stable/>`_) which allow almost
       arbitrary date tick specifications.  See `rrule example
-      <../examples/pylab_examples/date_demo_rrule.html>`_.
+      <../gallery/ticks_and_spines/date_demo_rrule.html>`_.
 
     * :class:`AutoDateLocator`: On autoscale, this class picks the best
       :class:`MultipleDateLocator` to set the view limits and the tick
@@ -112,9 +111,9 @@ Here all all the date formatters:
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-from matplotlib.externals import six
-from matplotlib.externals.six.moves import xrange, zip
-
+import six
+from six.moves import zip
+from matplotlib import rcParams
 import re
 import time
 import math
@@ -137,7 +136,7 @@ import matplotlib.cbook as cbook
 import matplotlib.ticker as ticker
 
 
-__all__ = ('date2num', 'num2date', 'drange', 'epoch2num',
+__all__ = ('date2num', 'num2date', 'num2timedelta', 'drange', 'epoch2num',
            'num2epoch', 'mx2num', 'DateFormatter',
            'IndexDateFormatter', 'AutoDateFormatter', 'DateLocator',
            'RRuleLocator', 'AutoDateLocator', 'YearLocator',
@@ -160,11 +159,12 @@ class _UTC(datetime.tzinfo):
         return datetime.timedelta(0)
 
     def tzname(self, dt):
-        return "UTC"
+        return str("UTC")
 
     def dst(self, dt):
         return datetime.timedelta(0)
 
+
 UTC = _UTC()
 
 
@@ -178,6 +178,7 @@ def _get_rc_timezone():
     import pytz
     return pytz.timezone(s)
 
+
 """
 Time-related constants.
 """
@@ -212,23 +213,24 @@ def _to_ordinalf(dt):
     days, preserving hours, minutes, seconds and microseconds.  Return value
     is a :func:`float`.
     """
-
-    if hasattr(dt, 'tzinfo') and dt.tzinfo is not None:
-        delta = dt.tzinfo.utcoffset(dt)
-        if delta is not None:
-            dt -= delta
+    # Convert to UTC
+    tzi = getattr(dt, 'tzinfo', None)
+    if tzi is not None:
+        dt = dt.astimezone(UTC)
+        tzi = UTC
 
     base = float(dt.toordinal())
-    if isinstance(dt, datetime.datetime):
-        # Get a datetime object at midnight in the same time zone as dt.
-        cdate = dt.date()
-        midnight_time = datetime.time(0, 0, 0, tzinfo=dt.tzinfo)
+
+    # If it's sufficiently datetime-like, it will have a `date()` method
+    cdate = getattr(dt, 'date', lambda: None)()
+    if cdate is not None:
+        # Get a datetime object at midnight UTC
+        midnight_time = datetime.time(0, tzinfo=tzi)
 
         rdt = datetime.datetime.combine(cdate, midnight_time)
-        td_remainder = _total_seconds(dt - rdt)
 
-        if td_remainder > 0:
-            base += td_remainder / SEC_PER_DAY
+        # Append the seconds as a fraction of a day
+        base += (dt - rdt).total_seconds() / SEC_PER_DAY
 
     return base
 
@@ -236,23 +238,6 @@ def _to_ordinalf(dt):
 # a version of _to_ordinalf that can operate on numpy arrays
 _to_ordinalf_np_vectorized = np.vectorize(_to_ordinalf)
 
-try:
-    # Available as a native method in Python >= 2.7.
-    _total_seconds = datetime.timedelta.total_seconds
-except AttributeError:
-    def _total_seconds(tdelta):
-        """
-        Alias providing support for datetime.timedelta.total_seconds() function
-        calls even in Python < 2.7.
-
-        The input `tdelta` is a datetime.timedelta object, and returns a float
-        containing the total number of seconds representing the `tdelta`
-        duration. For large durations (> 270 on most platforms), this loses
-        microsecond accuracy.
-        """
-        return (tdelta.microseconds +
-                (tdelta.seconds + tdelta.days * SEC_PER_DAY) * 1e6) * 1e-6
-
 
 def _from_ordinalf(x, tz=None):
     """
@@ -291,8 +276,7 @@ _from_ordinalf_np_vectorized = np.vectorize(_from_ordinalf)
 class strpdate2num(object):
     """
     Use this class to parse date strings to matplotlib datenums when
-    you know the date format string of the date you are parsing.  See
-    :file:`examples/load_demo.py`.
+    you know the date format string of the date you are parsing.
     """
     def __init__(self, fmt):
         """ fmt: any valid strptime format is supported """
@@ -309,7 +293,7 @@ class bytespdate2num(strpdate2num):
     """
     Use this class to parse date strings to matplotlib datenums when
     you know the date format string of the date you are parsing.  See
-    :file:`examples/load_demo.py`.
+    :file:`examples/misc/load_converter.py`.
     """
     def __init__(self, fmt, encoding='utf-8'):
         """
@@ -345,10 +329,10 @@ def datestr2num(d, default=None):
     d : string or sequence of strings
         The dates to convert.
 
-    default : datetime instance
-        The default date to use when fields are missing in `d`.
+    default : datetime instance, optional
+        The default date to use when fields are missing in *d*.
     """
-    if cbook.is_string_like(d):
+    if isinstance(d, six.string_types):
         dt = dateutil.parser.parse(d, default=default)
         return date2num(dt)
     else:
@@ -362,14 +346,23 @@ def datestr2num(d, default=None):
 
 def date2num(d):
     """
-    *d* is either a :class:`datetime` instance or a sequence of datetimes.
+    Converts datetime objects to Matplotlib dates.
 
-    Return value is a floating point number (or sequence of floats)
-    which gives the number of days (fraction part represents hours,
-    minutes, seconds) since 0001-01-01 00:00:00 UTC, *plus* *one*.
-    The addition of one here is a historical artifact.  Also, note
-    that the Gregorian calendar is assumed; this is not universal
-    practice.  For details, see the module docstring.
+    Parameters
+    ----------
+    d : :class:`datetime` or sequence of :class:`datetime`
+
+    Returns
+    -------
+    float or sequence of floats
+        Number of days (fraction part represents hours, minutes, seconds)
+        since 0001-01-01 00:00:00 UTC, plus one.
+
+    Notes
+    -----
+    The addition of one here is a historical artifact. Also, note that the
+    Gregorian calendar is assumed; this is not universal practice.
+    For details see the module docstring.
     """
     if not cbook.iterable(d):
         return _to_ordinalf(d)
@@ -383,6 +376,16 @@ def date2num(d):
 def julian2num(j):
     """
     Convert a Julian date (or sequence) to a matplotlib date (or sequence).
+
+    Parameters
+    ----------
+    k : float or sequence of floats
+        Julian date(s)
+
+    Returns
+    -------
+    float or sequence of floats
+        Matplotlib date(s)
     """
     if cbook.iterable(j):
         j = np.asarray(j)
@@ -391,7 +394,17 @@ def julian2num(j):
 
 def num2julian(n):
     """
-    Convert a matplotlib date (or sequence) to a Julian date (or sequence).
+    Convert a Matplotlib date (or sequence) to a Julian date (or sequence).
+
+    Parameters
+    ----------
+    n : float or sequence of floats
+        Matplotlib date(s)
+
+    Returns
+    -------
+    float or sequence of floats
+        Julian date(s)
     """
     if cbook.iterable(n):
         n = np.asarray(n)
@@ -400,18 +413,27 @@ def num2julian(n):
 
 def num2date(x, tz=None):
     """
-    *x* is a float value which gives the number of days
-    (fraction part represents hours, minutes, seconds) since
-    0001-01-01 00:00:00 UTC *plus* *one*.
-    The addition of one here is a historical artifact.  Also, note
-    that the Gregorian calendar is assumed; this is not universal
-    practice.  For details, see the module docstring.
+    Parameters
+    ----------
+    x : float or sequence of floats
+        Number of days (fraction part represents hours, minutes, seconds)
+        since 0001-01-01 00:00:00 UTC, plus one.
+    tz : string, optional
+        Timezone of *x* (defaults to rcparams TZ value).
 
-    Return value is a :class:`datetime` instance in timezone *tz* (default to
-    rcparams TZ value).
+    Returns
+    -------
+    :class:`datetime` or sequence of :class:`datetime`
+        Dates are returned in timezone *tz*
 
     If *x* is a sequence, a sequence of :class:`datetime` objects will
     be returned.
+
+    Notes
+    -----
+    The addition of one here is a historical artifact. Also, note that the
+    Gregorian calendar is assumed; this is not universal practice.
+    For details, see the module docstring.
     """
     if tz is None:
         tz = _get_rc_timezone()
@@ -424,6 +446,38 @@ def num2date(x, tz=None):
         return _from_ordinalf_np_vectorized(x, tz).tolist()
 
 
+def _ordinalf_to_timedelta(x):
+    return datetime.timedelta(days=x)
+
+
+_ordinalf_to_timedelta_np_vectorized = np.vectorize(_ordinalf_to_timedelta)
+
+
+def num2timedelta(x):
+    """
+    Converts number of days to a :class:`timdelta` object.
+    If *x* is a sequence, a sequence of :class:`timedelta` objects will
+    be returned.
+
+    Parameters
+    ----------
+    x : float, sequence of floats
+        Number of days (fraction part represents hours, minutes, seconds)
+
+    Returns
+    -------
+    :class:`timedelta` or list[:class:`timedelta`]
+
+    """
+    if not cbook.iterable(x):
+        return _ordinalf_to_timedelta(x)
+    else:
+        x = np.asarray(x)
+        if not x.size:
+            return x
+        return _ordinalf_to_timedelta_np_vectorized(x).tolist()
+
+
 def drange(dstart, dend, delta):
     """
     Return a date range as float Gregorian ordinals.  *dstart* and
@@ -432,7 +486,7 @@ def drange(dstart, dend, delta):
     """
     f1 = _to_ordinalf(dstart)
     f2 = _to_ordinalf(dend)
-    step = _total_seconds(delta) / SEC_PER_DAY
+    step = delta.total_seconds() / SEC_PER_DAY
 
     # calculate the difference between dend and dstart in times of delta
     num = int(np.ceil((f2 - f1) / step))
@@ -609,7 +663,7 @@ class IndexDateFormatter(ticker.Formatter):
 
     def __call__(self, x, pos=0):
         'Return the label for time *x* at position *pos*'
-        ind = int(round(x))
+        ind = int(np.round(x))
         if ind >= len(self.t) or ind <= 0:
             return ''
 
@@ -629,12 +683,14 @@ class AutoDateFormatter(ticker.Formatter):
     format string.  The default looks like this::
 
         self.scaled = {
-           365.0  : '%Y',
-           30.    : '%b %Y',
-           1.0    : '%b %d %Y',
-           1./24. : '%H:%M:%S',
-           1. / (24. * 60.): '%H:%M:%S.%f',
-           }
+            DAYS_PER_YEAR: rcParams['date.autoformat.year'],
+            DAYS_PER_MONTH: rcParams['date.autoformat.month'],
+            1.0: rcParams['date.autoformat.day'],
+            1. / HOURS_PER_DAY: rcParams['date.autoformat.hour'],
+            1. / (MINUTES_PER_DAY): rcParams['date.autoformat.minute'],
+            1. / (SEC_PER_DAY): rcParams['date.autoformat.second'],
+            1. / (MUSECONDS_PER_DAY): rcParams['date.autoformat.microsecond'],
+            }
 
 
     The algorithm picks the key in the dictionary that is >= the
@@ -686,26 +742,28 @@ class AutoDateFormatter(ticker.Formatter):
         self._tz = tz
         self.defaultfmt = defaultfmt
         self._formatter = DateFormatter(self.defaultfmt, tz)
-        self.scaled = {DAYS_PER_YEAR: '%Y',
-                       DAYS_PER_MONTH: '%b %Y',
-                       1.0: '%b %d %Y',
-                       1. / HOURS_PER_DAY: '%H:%M:%S',
-                       1. / (MINUTES_PER_DAY): '%H:%M:%S.%f'}
+        self.scaled = {DAYS_PER_YEAR: rcParams['date.autoformatter.year'],
+                       DAYS_PER_MONTH: rcParams['date.autoformatter.month'],
+                       1.0: rcParams['date.autoformatter.day'],
+                       1. / HOURS_PER_DAY: rcParams['date.autoformatter.hour'],
+                       1. / (MINUTES_PER_DAY):
+                           rcParams['date.autoformatter.minute'],
+                       1. / (SEC_PER_DAY):
+                           rcParams['date.autoformatter.second'],
+                       1. / (MUSECONDS_PER_DAY):
+                           rcParams['date.autoformatter.microsecond']}
 
     def __call__(self, x, pos=None):
         locator_unit_scale = float(self._locator._get_unit())
-        fmt = self.defaultfmt
-
         # Pick the first scale which is greater than the locator unit.
-        for possible_scale in sorted(self.scaled):
-            if possible_scale >= locator_unit_scale:
-                fmt = self.scaled[possible_scale]
-                break
+        fmt = next((fmt for scale, fmt in sorted(self.scaled.items())
+                    if scale >= locator_unit_scale),
+                   self.defaultfmt)
 
         if isinstance(fmt, six.string_types):
             self._formatter = DateFormatter(fmt, self._tz)
             result = self._formatter(x, pos)
-        elif six.callable(fmt):
+        elif callable(fmt):
             result = fmt(x, pos)
         else:
             raise TypeError('Unexpected type passed to {0!r}.'.format(self))
@@ -729,6 +787,9 @@ class rrulewrapper(object):
             return self.__dict__[name]
         return getattr(self._rrule, name)
 
+    def __setstate__(self, state):
+        self.__dict__.update(state)
+
 
 class DateLocator(ticker.Locator):
     """
@@ -819,30 +880,17 @@ class RRuleLocator(DateLocator):
         # We need to cap at the endpoints of valid datetime
         try:
             start = vmin - delta
-        except ValueError:
+        except (ValueError, OverflowError):
             start = _from_ordinalf(1.0)
 
         try:
             stop = vmax + delta
-        except ValueError:
+        except (ValueError, OverflowError):
             # The magic number!
             stop = _from_ordinalf(3652059.9999999)
 
         self.rule.set(dtstart=start, until=stop)
 
-        # estimate the number of ticks very approximately so we don't
-        # have to do a very expensive (and potentially near infinite)
-        # 'between' calculation, only to find out it will fail.
-        nmax, nmin = date2num((vmax, vmin))
-        estimate = (nmax - nmin) / (self._get_unit() * self._get_interval())
-        # This estimate is only an estimate, so be really conservative
-        # about bailing...
-        if estimate > self.MAXTICKS * 2:
-            raise RuntimeError(
-                'RRuleLocator estimated to generate %d ticks from %s to %s: '
-                'exceeds Locator.MAXTICKS * 2 (%d) ' % (estimate, vmin, vmax,
-                                                        self.MAXTICKS * 2))
-
         dates = self.rule.between(vmin, vmax, True)
         if len(dates) == 0:
             return date2num([vmin, vmax])
@@ -985,8 +1033,7 @@ class AutoDateLocator(DateLocator):
                 # Assume we were given an integer. Use this as the maximum
                 # number of ticks for every frequency and create a
                 # dictionary for this
-                self.maxticks = dict(zip(self._freqs,
-                                         [maxticks] * len(self._freqs)))
+                self.maxticks = dict.fromkeys(self._freqs, maxticks)
         self.interval_multiples = interval_multiples
         self.intervald = {
             YEARLY:   [1, 2, 4, 5, 10, 20, 40, 50, 100, 200, 400, 500,
@@ -1058,8 +1105,8 @@ class AutoDateLocator(DateLocator):
         numDays = tdelta.days   # Avoids estimates of days/month, days/year
         numHours = (numDays * HOURS_PER_DAY) + delta.hours
         numMinutes = (numHours * MIN_PER_HOUR) + delta.minutes
-        numSeconds = np.floor(_total_seconds(tdelta))
-        numMicroseconds = np.floor(_total_seconds(tdelta) * 1e6)
+        numSeconds = np.floor(tdelta.total_seconds())
+        numMicroseconds = np.floor(tdelta.total_seconds() * 1e6)
 
         nums = [numYears, numMonths, numDays, numHours, numMinutes,
                 numSeconds, numMicroseconds]
@@ -1175,7 +1222,7 @@ class YearLocator(DateLocator):
         ymax = self.base.ge(vmax.year)
 
         ticks = [vmin.replace(year=ymin, **self.replaced)]
-        while 1:
+        while True:
             dt = ticks[-1]
             if dt.year >= ymax:
                 return date2num(ticks)
@@ -1263,6 +1310,8 @@ class DayLocator(RRuleLocator):
 
         Default is to tick every day of the month: ``bymonthday=range(1,32)``
         """
+        if not interval == int(interval) or interval < 1:
+            raise ValueError("interval must be an integer greater than 0")
         if bymonthday is None:
             bymonthday = range(1, 32)
         elif isinstance(bymonthday, np.ndarray):
@@ -1399,7 +1448,7 @@ def _close_to_dt(d1, d2, epsilon=5):
     Assert that datetimes *d1* and *d2* are within *epsilon* microseconds.
     """
     delta = d2 - d1
-    mus = abs(_total_seconds(delta) * 1e6)
+    mus = abs(delta.total_seconds() * 1e6)
     assert mus < epsilon
 
 
