@@ -22,7 +22,8 @@ __all__ = ['streamplot']
 
 def streamplot(axes, x, y, u, v, density=1, linewidth=None, color=None,
                cmap=None, norm=None, arrowsize=1, arrowstyle='-|>',
-               minlength=0.1, transform=None, zorder=None, start_points=None):
+               minlength=0.1, transform=None, zorder=None, start_points=None,
+               maxlength=4.0, integration_direction='both'):
     """Draws streamlines of a vector flow.
 
     *x*, *y* : 1d arrays
@@ -58,6 +59,10 @@ def streamplot(axes, x, y, u, v, density=1, linewidth=None, color=None,
         In data coordinates, the same as the ``x`` and ``y`` arrays.
     *zorder* : int
         any number
+    *maxlength* : float
+        Maximum length of streamline in axes coordinates.
+    *integration_direction* : ['forward', 'backward', 'both']
+        Integrate the streamline in forward, backward or both directions.
 
     Returns:
 
@@ -95,6 +100,15 @@ def streamplot(axes, x, y, u, v, density=1, linewidth=None, color=None,
     line_kw = {}
     arrow_kw = dict(arrowstyle=arrowstyle, mutation_scale=10 * arrowsize)
 
+    if integration_direction not in ['both', 'forward', 'backward']:
+        errstr = ("Integration direction '%s' not recognised. "
+                  "Expected 'both', 'forward' or 'backward'." %
+                  integration_direction)
+        raise ValueError(errstr)
+
+    if integration_direction == 'both':
+        maxlength /= 2.
+
     use_multicolor_lines = isinstance(color, np.ndarray)
     if use_multicolor_lines:
         if color.shape != grid.shape:
@@ -126,7 +140,8 @@ def streamplot(axes, x, y, u, v, density=1, linewidth=None, color=None,
     u = np.ma.masked_invalid(u)
     v = np.ma.masked_invalid(v)
 
-    integrate = get_integrator(u, v, dmap, minlength)
+    integrate = get_integrator(u, v, dmap, minlength, maxlength,
+                               integration_direction)
 
     trajectories = []
     if start_points is None:
@@ -137,12 +152,21 @@ def streamplot(axes, x, y, u, v, density=1, linewidth=None, color=None,
                 if t is not None:
                     trajectories.append(t)
     else:
+        sp2 = np.asanyarray(start_points, dtype=float).copy()
+
+        # Check if start_points are outside the data boundaries
+        for xs, ys in sp2:
+            if not (grid.x_origin <= xs <= grid.x_origin + grid.width
+                    and grid.y_origin <= ys <= grid.y_origin + grid.height):
+                raise ValueError("Starting point ({}, {}) outside of data "
+                                 "boundaries".format(xs, ys))
+
         # Convert start_points from data to array coords
         # Shift the seed points from the bottom left of the data so that
         # data2grid works properly.
-        sp2 = np.asanyarray(start_points, dtype=np.float).copy()
-        sp2[:, 0] += np.abs(x[0])
-        sp2[:, 1] += np.abs(y[0])
+        sp2[:, 0] -= grid.x_origin
+        sp2[:, 1] -= grid.y_origin
+
         for xs, ys in sp2:
             xg, yg = dmap.data2grid(xs, ys)
             t = integrate(xg, yg)
@@ -163,8 +187,9 @@ def streamplot(axes, x, y, u, v, density=1, linewidth=None, color=None,
         tgx = np.array(t[0])
         tgy = np.array(t[1])
         # Rescale from grid-coordinates to data-coordinates.
-        tx = np.array(t[0]) * grid.dx + grid.x_origin
-        ty = np.array(t[1]) * grid.dy + grid.y_origin
+        tx, ty = dmap.grid2data(*np.array(t))
+        tx += grid.x_origin
+        ty += grid.y_origin
 
         points = np.transpose([tx, ty]).reshape(-1, 1, 2)
         streamlines.extend(np.hstack([points[:-1], points[1:]]))
@@ -244,8 +269,8 @@ class DomainMap(object):
         self.x_mask2grid = 1. / self.x_grid2mask
         self.y_mask2grid = 1. / self.y_grid2mask
 
-        self.x_data2grid = grid.nx / grid.width
-        self.y_data2grid = grid.ny / grid.height
+        self.x_data2grid = 1. / grid.dx
+        self.y_data2grid = 1. / grid.dy
 
     def grid2mask(self, xi, yi):
         """Return nearest space in mask-coords from given grid-coords."""
@@ -258,6 +283,9 @@ class DomainMap(object):
     def data2grid(self, xd, yd):
         return xd * self.x_data2grid, yd * self.y_data2grid
 
+    def grid2data(self, xg, yg):
+        return xg / self.x_data2grid, yg / self.y_data2grid
+
     def start_trajectory(self, xg, yg):
         xm, ym = self.grid2mask(xg, yg)
         self.mask._start_trajectory(xm, ym)
@@ -385,7 +413,7 @@ class TerminateTrajectory(Exception):
 # Integrator definitions
 #========================
 
-def get_integrator(u, v, dmap, minlength):
+def get_integrator(u, v, dmap, minlength, maxlength, integration_direction):
 
     # rescale velocity onto grid-coordinates for integrations.
     u, v = dmap.data2grid(u, v)
@@ -419,14 +447,27 @@ def get_integrator(u, v, dmap, minlength):
         resulting trajectory is None if it is shorter than `minlength`.
         """
 
-        dmap.start_trajectory(x0, y0)
-        sf, xf_traj, yf_traj = _integrate_rk12(x0, y0, dmap, forward_time)
-        dmap.reset_start_point(x0, y0)
-        sb, xb_traj, yb_traj = _integrate_rk12(x0, y0, dmap, backward_time)
-        # combine forward and backward trajectories
-        stotal = sf + sb
-        x_traj = xb_traj[::-1] + xf_traj[1:]
-        y_traj = yb_traj[::-1] + yf_traj[1:]
+        stotal, x_traj, y_traj = 0., [], []
+
+        try:
+            dmap.start_trajectory(x0, y0)
+        except InvalidIndexError:
+            return None
+        if integration_direction in ['both', 'backward']:
+            s, xt, yt = _integrate_rk12(x0, y0, dmap, backward_time, maxlength)
+            stotal += s
+            x_traj += xt[::-1]
+            y_traj += yt[::-1]
+
+        if integration_direction in ['both', 'forward']:
+            dmap.reset_start_point(x0, y0)
+            s, xt, yt = _integrate_rk12(x0, y0, dmap, forward_time, maxlength)
+            if len(x_traj) > 0:
+                xt = xt[1:]
+                yt = yt[1:]
+            stotal += s
+            x_traj += xt
+            y_traj += yt
 
         if stotal > minlength:
             return x_traj, y_traj
@@ -437,7 +478,7 @@ def get_integrator(u, v, dmap, minlength):
     return integrate
 
 
-def _integrate_rk12(x0, y0, dmap, f):
+def _integrate_rk12(x0, y0, dmap, f, maxlength):
     """2nd-order Runge-Kutta algorithm with adaptive step size.
 
     This method is also referred to as the improved Euler's method, or Heun's
@@ -513,7 +554,7 @@ def _integrate_rk12(x0, y0, dmap, f):
                 dmap.update_trajectory(xi, yi)
             except InvalidIndexError:
                 break
-            if (stotal + ds) > 2:
+            if (stotal + ds) > maxlength:
                 break
             stotal += ds
 
@@ -558,14 +599,14 @@ def interpgrid(a, xi, yi):
 
     Ny, Nx = np.shape(a)
     if isinstance(xi, np.ndarray):
-        x = xi.astype(np.int)
-        y = yi.astype(np.int)
+        x = xi.astype(int)
+        y = yi.astype(int)
         # Check that xn, yn don't exceed max index
         xn = np.clip(x + 1, 0, Nx - 1)
         yn = np.clip(y + 1, 0, Ny - 1)
     else:
-        x = np.int(xi)
-        y = np.int(yi)
+        x = int(xi)
+        y = int(yi)
         # conditional is faster than clipping for integers
         if x == (Nx - 2):
             xn = x
